import java.util.Arrays;

/**
 * Renderer.
 * 
 * Wandelt in seiner toString Methode die gespeicherte {@link SnowflakeList} in
 * einen darstellbaren String um. Zusätzlich wird im Hintergrund ein
 * Weihnachtsbaum gezeichnet. Auf diesem können Schneeflocken liegen bleiben.
 * 
 * @author Stephan Bauer
 * @author Thomas Gradisnik
 * @version 1.0
 * 
 */

public class Renderer {

    private SnowflakeList list;
    private int width;
    private int height;

    /**
     * In diesem Array werden liegen gebliebene Schneeflocken mit ihrem Symbol
     * gespeichert
     */
    private char[][] form;

    /** Das zu zeichnende ChristmasTree Objekt */
    private ChristmasTree christmasTree;

    /**
     * Konstruktor.
     * 
     * Initialisiert den Renderer mit einer Liste der Schneeflocken und einer
     * Höhe und einer Breite die zum Erzeugen des Strings verwendet wird.
     * 
     * @param list
     *            Die Liste der Schneeflocken
     * @param width
     *            Die Breite des Fensters
     * @param height
     *            Die Höhe des Fensters
     */
    public Renderer(SnowflakeList list, int width, int height) {

	this.list = list;
	this.width = width;
	this.height = height;
	
	this.form = new char[height][width];
	for (int i = 0; i < this.form.length; i++) {
	    Arrays.fill(this.form[i], ' ');
	}

	this.christmasTree = new ChristmasTree(width, height);
		
    }

    /**
     * Erzeugt eine darstellbare Repräsentation der gespeicherten
     * Schneeflockenliste.
     * 
     * Dafür wird der gesamte Bereich des width * height großen Rechtecks
     * durchgegangen und für jede Stelle abgefragt, ob es in der SnowflakeList
     * an dieser Stelle ein Element gibt. Ist an dieser Stelle keine
     * Schneeflocke, so wird überprüft ob an dieser Stelle bereits eine
     * Schneeflocke liegt und gezeichnet werden soll, oder ob an der Stelle ein
     * Teil des Weihnachtsbaum gezeichnet werden soll.
     */
    public String toString() {

	String result = "";
	
	for (int j = 0; j < height; j++) {

	    result += "\n";

	    for (int i = 0; i < width; i++) {

		if (null != list.snowflakeAt(i, j)) {

		    // Es existiert an dieser Stelle eine Schneeflocke, daher
		    // wird diese gezeichnet
		    Snowflake snowflake = list.snowflakeAt(i, j);
		    result += snowflake;

		    if (this.form[j][i] == ' ' && this.christmasTree.valueAt(i, j)) {
			
			// Die Schneeflocke befindet sich auf dem Christbaum und
			// soll dort liegen bleiben, daher wird sie in 
			// das form Array gespeichert und aus der
			// SnowflakeList entfernt.
			this.form[j][i] = snowflake.toString().charAt(0);
			list.revive(snowflake);
		    
		    }

		} else {

		    if (this.form[j][i] != ' ') {

			result += this.form[j][i];

		    } else {
			
			// Weihnachtsbaum zeichnen?
			if (this.christmasTree.valueAt(i, j)) {
			    result += "-";
			} else {
			    result += " ";
			}

		    }

		} 

	    } 

	} 

	return result;

    }

}
