import java.util.HashMap;

/**
 * Repräsentiert einen Parkplatz.
 */
public class Parkplatz {
        
        private HashMap<String, Integer> map;
        private int stellplaetze;
        
        /**
         * Initialisiert den Parkplatz mit einer entsprechenden Anzahl
         * an freien Plätzen und 0 belegten Plätzen.
         * @param stellplaetze 
         */
        public Parkplatz(int stellplaetze) {
        
                this.stellplaetze = stellplaetze;
                this.map = new HashMap<String, Integer>();

        }
        
        /**
         * @return liefert die Gesamtanzahl der Stellplätze auf diesem
         * Parkplatz.
         */
        public int stellplaetze() {
        
                return stellplaetze;
                
        }
        
        /**
         * Gibt an, ob der Stellplatz i belegt ist.
         * @param i der Index des Stellplatzes
         * @return true, wenn der Stellplatz i belegt ist, sonst false
         */
        public boolean istBelegt(int i) {
                
                return map.containsValue(i);
                
        }
        
        /**
         * Gibt an, ob das KFZ mit dem angegebenen Kennzeichen auf dem Parkplatz
         * abgestellt wurde.
         * @param kennzeichen das Kennzeichen des KFZ
         * @return true, wenn der das KFZ mit dem angegebenen Kennzeichen auf 
         * 	einem beliebigen Platz steht, sonst false
         */
        public boolean istAbgestellt(String kennzeichen) {
                
                return map.containsKey(kennzeichen);
                
        }
        
        /**
         * Stellt das KFZ mit dem angegebenen Kennzeichen auf dem 
         * angegebenen Stellplatz ab.
         * @param kennzeichen das Kennzeichen des KFZ
         * @param i der Index des Stellplatzes
         */
        public void stellen(String kennzeichen, int i) {
                
                if (istBelegt(i)) {
                        
                        return;
                
                }
                
                map.put(kennzeichen, i);
                
        }
        
        /**
         * Entfernt das KFZ mit dem angebebenen Kennzeichen 
         * @param kennzeichen das Kennzeichen des zu entfernenden KFZ
         */
        public void entfernen(String kennzeichen) {
        
                map.remove(kennzeichen);
        
        }        
        
        
        /**
         * @return eine lesbare Repräsentation des Parkplatzes, mit 
         * allen belegten Plätzen.
         */
        public String toString() {
        
                return map.toString() + "\n" + stellplaetze + " Stellplätze insgesamt";
        
        }
        
}
