import java.util.*;

/** 
* Train mit Iterator
*/

public class Train implements Iterable<Car> {
	
	private Car head;
	private Car last;
	private int modCount;
	
	public Train(Locomotive head) {
		
		this.head = head;
		this.last = head;
		modCount = 0;
		
	} 
	
	public Iterator<Car> iterator() {
		
		return new CarIterator();
		
	}
	
	private class CarIterator implements Iterator<Car> {
		
		Car next;
		Car lastReturned = head; 
		//head ist nun ein "Dummyknoten", um Randflle
		//einfacher zu machen. wird nicht ausgegeben.
		
		Car previousToLastReturned = null;
		private int knownMods;
		
		CarIterator() {
			
			this.next = Train.this.head.next();
			knownMods = Train.this.modCount;
			
		}
		
		public boolean hasNext() {
			
			return next != null;
			
		}
		
		public Car next() {
			
			Car c = next;
			
			if (lastReturned != null) {
			
				previousToLastReturned = lastReturned;
				
			}
			
			lastReturned = c;
			next = next.next();
			
			return c;
			
		}
		
		/**
		* Remove the most recently returned element from the list.
		*
		* @throws ConcurrentModificationException if the list was modified
		* @throws IllegalStateException if there was no last element
		*/
		public void remove() {
			
			//throw new UnsupportedOperationException("not supported");
			
			if (knownMods != Train.this.modCount) {
			
				throw new ConcurrentModificationException();
				//toDo: sollte auch an anderen Stellen 
				//geworfen werden
				
			}
			
			if (lastReturned == null) {
			
				throw new IllegalStateException();
			
			}
		
			if (previousToLastReturned == null) {
				
				 lastReturned.next().connectTo(Train.this.head);
			
			} else {
			
				 lastReturned.next().connectTo(previousToLastReturned);
			
			}
			
			knownMods++;
			Train.this.modCount++;
			lastReturned = null;
			
			
		}
		
	}
	
	public boolean equals(Object o) {
		
		if (o == null) {
			
			return false;
			
		}
		
		if (this.getClass() != o.getClass()) {
			
			return false;
			
		}
		
		
		Train other = (Train) o;
		
		return this.head.equals(other.head);
		
	}
	
	private Car getLast() {
		
		return last;
		
	}
	
	public void add(Car toAppend) {
		
		toAppend.connectTo(last);
		this.last = toAppend;
		modCount++;
		
	}
	
	public boolean hasOpenDiningCar() {
		
		return head.next().hasOpenDiningCar();
		
	}
	
	public String toString() {
		
		String s = head.toString() + "\n";
		
		for(Car c: this) {  //Nutzen des Iterators
			
			s += c + "\n";
			
		}
		
		return s + (this.hasOpenDiningCar() ? "hat geoeffneten Speisewagen." : "");
		
		
	}
	
}
